package HelloClient;

import java.io.*;
import java.rmi.*;
import HelloInterface.*;

//////////////////////////
//we need import this lib for CyclicBarrier
import java.util.concurrent.CyclicBarrier;

public class HelloClient 
{
	public static HelloWorldInterface helloRef;
	public static final int MaxThreadNumber=80;
	//////////////////////////////////////
	//here we declare two equal number of array of threads and initialize them
	public static IncThread[] incThreadArray=new IncThread[MaxThreadNumber];
	public static DecThread[] decThreadArray=new DecThread[MaxThreadNumber];

	//////////////////////////////////
	//a small skill to stop all created threads run one by one is to use 
	//a barrier to "stop them until all threads are ready
	////////////////////////////////////////////////
	public static CyclicBarrier barrier=new CyclicBarrier(MaxThreadNumber*2);

	public static void main(String args[]) 
	{
	  try {
		 int RMIPort=1099;         
		 String hostName="localhost";
  
		 String registryURL = 
			"rmi://" + hostName+ ":" + RMIPort + "/HelloInterface";  
		 // find the remote object and cast it to an interface object
		 helloRef= (HelloWorldInterface)Naming.lookup(registryURL);
		 System.out.println("Lookup completed " );
		 // invoke the remote method
		
		
		/////////////////////////////////////////////////////////////
		//here is our testing function to create equal number of threads to do inc and dec of price.

		//////////////////////////////////////
		//
		
	

		System.out.println("before thread testing starts, let's see the balance of myPrice at server side");
		System.out.println(helloRef.sayHello("Bob"));

		System.out.println("Since we are using equal number of threads to run increasing and decreasing, if"); 
		System.out.println(" our system is properly synchronized, after testing the balance should remain unchanged.");
		
		////////////////////////////////////
		//since we have a "barrier" setup, the threads won't start until all are ready
		for (int i=0; i<MaxThreadNumber; i++)
		{
			incThreadArray[i]=new IncThread();
			incThreadArray[i].start();
			decThreadArray[i]=new DecThread();
			decThreadArray[i].start();
		}
		
		////////////////////////////////////
		//Here our primary thread or main thread must wait all threads to finish
		for (int i=0; i<MaxThreadNumber; i++)
		{			
			incThreadArray[i].join();		
			decThreadArray[i].join();
		}
		//then we print our result.
		System.out.println("Now the testing ends, let's check if the balance is unchanged"+helloRef.sayHello("Bob"));


		 
	  } // end try 
	  catch (Exception e) {
		 System.out.println("Exception in HelloClient: " + e);
	  } 
	} //end main

	
	///////////////////////////////////////////////////
	//here we define two helper class which is thread and they do increasing, decreasing price.
	/////////////////////////////////////////////////


	public static class IncThread extends Thread
	{		
		public void run()
		{
			try
			{
				barrier.await();	
				helloRef.incPrice();
			}
			catch(Exception er)
			{
				System.out.println(er);
			}
		}
	}

	public static class DecThread extends Thread
	{
		public void run()
		{
			try
			{
				barrier.await();	
				helloRef.decPrice();
			}
			catch(Exception er)
			{
				System.out.println(er);
			}
		}
	}

			
}//end class
